import 'dart:math';
import 'dart:typed_data';

import 'package:encrypt/encrypt.dart';

const String VBDL_HEADER_ENCKEY = "63c103808e1be8c71e6578acc766c9cb";
const String VBDL_DATA_ENCKEY = "d6f1817e264f265509e5edf0f73e96c3";
const String VBDL_ASSETS_KEY = "assets.pak";

const AESMode VBDL_HEADER_ENCMODE = AESMode.ecb;
const AESMode VBDL_INDEX_ENCMODE = AESMode.cfb64;
const AESMode VBDL_DATA_ENCMODE = AESMode.sic;

class VBDLHeader {
  final int _random0; // 32
  final int createTime; // 32
  final int _random1; // 32
  final int indexStart; // 32
  final int _random2; // 32
  final int indexLength; // 32
  final int _random3; // 32
  final int dataStart; // 32
  final int _random4; // 32
  final int dataLength; // 32
  final int _random5; // 32
  final int reserve; // 32

  static const int ENCRYPTED_LEN = 64;

  // 原始48字节 => 加密后64字节
  VBDLHeader(
      this._random0,
      this.createTime,
      this._random1,
      this.indexStart,
      this._random2,
      this.indexLength,
      this._random3,
      this.dataStart,
      this._random4,
      this.dataLength,
      this._random5,
      this.reserve);

  factory VBDLHeader.fromBytes(Uint8List bytes) {
    final u32s = Uint32List.view(bytes.buffer);
    return VBDLHeader(
      u32s[0],
      u32s[1],
      u32s[2],
      u32s[3],
      u32s[4],
      u32s[5],
      u32s[6],
      u32s[7],
      u32s[8],
      u32s[9],
      u32s[10],
      u32s[11],
    );
  }

  Uint8List toBytes() {
    final u32s = Uint32List(12);
    u32s[0] = _random0;
    u32s[1] = createTime;
    u32s[2] = _random1;
    u32s[3] = indexStart;
    u32s[4] = _random2;
    u32s[5] = indexLength;
    u32s[6] = _random3;
    u32s[7] = dataStart;
    u32s[8] = _random4;
    u32s[9] = dataLength;
    u32s[10] = _random5;
    u32s[11] = reserve;
    return Uint8List.view(u32s.buffer);
  }

  VBDLHeader.make(
      this.indexStart, this.indexLength, this.dataStart, this.dataLength)
      : createTime = DateTime.now().millisecondsSinceEpoch,
        reserve = 0,
        _random0 = Random().nextInt((1 << 32)),
        _random1 = Random().nextInt((1 << 32)),
        _random2 = Random().nextInt((1 << 32)),
        _random3 = Random().nextInt((1 << 32)),
        _random4 = Random().nextInt((1 << 32)),
        _random5 = Random().nextInt((1 << 32));
}
