// ignore_for_file: unnecessary_null_comparison

import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:crypto/crypto.dart';
import 'package:encrypt/encrypt.dart';

import 'config.dart';

Uint8List? _decrypt(Key key, Uint8List bytes,
    {AESMode mode: AESMode.sic, IV? iv}) {
  iv = iv ?? IV.fromLength(key.bytes.length);
  try {
    return AES(key, mode: mode).decrypt(Encrypted(bytes), iv: iv);
  } catch (e) {
    return null;
  }
}

Key _makeKey(String key) {
  final digest = md5.convert(utf8.encode(key));
  return Key(Uint8List.fromList(digest.bytes));
}

final Key _headerDecKey = _makeKey(VBDL_HEADER_ENCKEY);
final Key _dataDecKey = _makeKey(VBDL_DATA_ENCKEY);
Key? _indexDecKey;

void bindIndexDecKey(String key) {
  assert(_indexDecKey == null && key != null);
  if (_indexDecKey != null) return;
  _indexDecKey = _makeKey(key);
}

VBDLHeader decryptHeader(Uint8List bytes) {
  final data = _decrypt(_headerDecKey, bytes, mode: VBDL_HEADER_ENCMODE);
  return VBDLHeader.fromBytes(data!);
}

Map<String, List<int>>? decryptIndex(Uint8List bytes) {
  assert(_indexDecKey != null);
  if (_indexDecKey == null) return null;

  bytes = _decrypt(_indexDecKey!, bytes, mode: VBDL_INDEX_ENCMODE)!;
  if (bytes == null) return null;
  final data = zlib.decode(bytes);
  if (data == null) return null;

  final string = String.fromCharCodes(data);

  Map<String, dynamic> map = json.decode(string);
  Map<String, List<int>> indexMap = {};
  map.forEach((k, v) {
    List<dynamic> pp = v;
    List<int> vv = [];
    assert(pp.length == 3);
    for (var i in pp) {
      assert(i is int);
      vv.add(i);
    }
    indexMap[k] = vv;
  });
  return indexMap;
}

List<int> decryptData(Uint8List bytes, int random) {
  List<int> lists = md5.convert(utf8.encode("$random")).bytes;
  IV iv = IV(Uint8List.fromList(lists));
  bytes = _decrypt(_dataDecKey, bytes, mode: VBDL_DATA_ENCMODE, iv: iv)!;
  return zlib.decode(bytes);
}
